# -*- coding: utf-8 -*-
"""
Created on Sat Mar  1 08:38:17 2025

@author: mstep
"""

def binary_contrast_enhancement(image, cutoff):
    '''Perform binary contrast enhancement.'''
    # Convert the cutoff from a fraction to an absolute value.
    cutoff = int(3 * 255 * cutoff)

    # Make a copy so we don't mess up the original.
    image = image.copy()

    # Convert to RGB if necessary.
    if image.mode != 'RGB':
        image = image.convert('RGB')

    # Process the copy's pixels.
    pixels = image.load()
    for y in range(image.height):
        for x in range(image.width):
            (r, g, b) = pixels[x, y]
            if r + g + b < cutoff:
                pixels[x, y] = (0, 0, 0)
            else:
                pixels[x, y] = (255, 255, 255)

    # Return the result image.
    return image

#%%

import tkinter as tk
from tkinter import messagebox
from tkinter import filedialog as fd
from PIL import Image, ImageTk

class LoadImageApp:
    def __init__(self):
        self.original_image = None
        self.original_photo_image = None
        self.result_image = None

        self.window = tk.Tk()

        self.window.title('pil_binary_contrast_enhancement')
        self.window.protocol('WM_DELETE_WINDOW', self.kill_callback)
        self.window.geometry('451x441')

        # Build the menu.
        self.build_menus()

        # Make the slider.
        frame = tk.Frame(self.window)
        frame.pack(side=tk.TOP, expand=False, fill=tk.X, anchor=tk.NW)

        label = tk.Label(frame, text='Cutoff:')
        label.pack(side=tk.LEFT, anchor=tk.SW)

        self.cutoff_var = tk.DoubleVar(value=0.5)
        scale = tk.Scale(frame, variable=self.cutoff_var,
            from_=0.0, to_=1.0, resolution=0.01,
            orient=tk.HORIZONTAL, command=lambda _: self.show_images())
        scale.pack(side=tk.LEFT, expand=True, fill=tk.X)

        # Create the Canvases.
        frame = tk.Frame(self.window)
        frame.pack(side=tk.TOP, expand=True, fill=tk.BOTH)

        self.original_canvas = tk.Canvas(frame, bg='white',
            borderwidth=0, highlightthickness=0, width=10, height=10)
        self.original_canvas.pack(side=tk.LEFT, fill=tk.BOTH, expand=True,
            anchor=tk.NW, padx=2, pady=2)

        self.result_canvas = tk.Canvas(frame, bg='white',
            borderwidth=0, highlightthickness=0, width=10, height=10)
        self.result_canvas.pack(side=tk.LEFT, fill=tk.BOTH, expand=True,
            anchor=tk.NW, padx=2, pady=2)

        # Display the window.
        self.window.focus_force()
        self.window.mainloop()

    def build_menus(self):
        '''Build the menu bar.'''
        menu_bar = tk.Menu(self.window)
        self.window.config(menu=menu_bar)
        self.build_file_menu(menu_bar)

    def build_file_menu(self, menu_bar):
        '''Build the File menu.'''
        self.file_menu = tk.Menu(menu_bar, tearoff=False)
        menu_bar.add_cascade(label='File', menu=self.file_menu)

        # Open.
        self.open_photo_image = ImageTk.PhotoImage(file='open.png')
        self.file_menu.add_command(label='Open...', accelerator='Ctrl+O',
            compound='left',
            command=self.mnu_open, image=self.open_photo_image)
        self.window.bind('<Control-o>', lambda _: self.mnu_open())

        # Save As.
        self.save_as_photo_image = ImageTk.PhotoImage(file='save_as.png')
        self.file_menu.add_command(label='Save As...', accelerator='Ctrl+S',
            compound='left',
            command=self.mnu_save_as, image=self.save_as_photo_image)
        self.window.bind('<Control-s>', lambda _: self.mnu_save_as())

        # Save As is disabled until we load an image.
        self.file_menu.entryconfig('Save As...', state='disabled')

        self.file_menu.add_separator()

        # Exit
        self.exit_photo_image = ImageTk.PhotoImage(file='exit.png')
        self.file_menu.add_command(label='Exit', compound='left',
            command=self.mnu_exit, image=self.exit_photo_image)

    def mnu_open(self):
        self.original_image = self.get_image_file('Load Image')
        self.original_photo_image = None
        self.show_images()

    def mnu_save_as(self):
        '''Save a file.'''
        filetypes = (
            ('Image Files', '*.png;*.jpg;*.bmp;*.gif;*.tif'),
            ('All files', '*.*')
        )
        filename = fd.asksaveasfilename(filetypes=filetypes)
        if len(filename) > 0:
            self.result_image.save(filename)

    def get_image_file(self, title):
        '''
        Let the user pick an image file.
        Then load and return it in an RGB PIL Image.
        '''
        filetypes = (
            ('Image Files', '*.png;*.jpg;*.bmp;*.gif;*.tif'),
            ('All files', '*.*')
        )
        filename = fd.askopenfilename(title=title, filetypes=filetypes)
        if not filename: return None

        try:
            return Image.open(filename)
        except Exception as e:
            messagebox.showinfo('Open Error', f'Error loading image file.\n{e}')
            return None

    def show_images(self):
        '''Display the image.'''
        # If we hvae no original image loaded, do nothing.
        if self.original_image is None: return

        # If we have not yet displayed the original image, do so.
        if self.original_photo_image is None:
            # Convert into a PhotoImage.
            self.original_photo_image = ImageTk.PhotoImage(self.original_image)
            self.original_canvas.delete(tk.ALL)
            self.original_canvas.create_image(0, 0, anchor=tk.NW,
                image=self.original_photo_image)

            # Enable the Save As menu item.
            self.file_menu.entryconfig('Save As...', state='normal')

        # Perform binary contrast enhancement.
        self.result_image = binary_contrast_enhancement(
            self.original_image, self.cutoff_var.get())

        # Display the result.
        self.result_photo_image = ImageTk.PhotoImage(self.result_image)
        self.result_canvas.delete(tk.ALL)
        self.result_canvas.create_image(0, 0, anchor=tk.NW,
            image=self.result_photo_image)

    def mnu_exit(self):
        print('Exit')
        self.kill_callback()

    def kill_callback(self):
        # Destroy the tkinter window.
        self.window.destroy()

# In[19]:
LoadImageApp()
print('Done')
